@@ -16,6 +16,7 @@ from almdrlib.config import Config
 from almdrlib.region import Region
 from almdrlib.client import Client
 import alsdkdefs
+from functools import lru_cache
 
 logger = logging.getLogger(__name__)
 
@@ -202,9 +203,7 @@ class Session():
         """
         requests lib auth module callback
         """
-        if self._token is None:
-            self._authenticate()
-        r.headers["x-aims-auth-token"] = self._token
+        r.headers["x-aims-auth-token"] = self.token
         return r
 
     def client(self, service_name, version=None, *args, **kwargs):
@@ -246,7 +245,19 @@ class Session():
             " class instance")
         return _client
 
+    @lru_cache(maxsize=128)
     def get_url(self, service_name, account_id=None):
+        """
+        Lookup account-specific URL prefix for a service
+
+        The host portion of each service URL can vary, based on the account_id of the
+        request. This function looks up the correct URL, via a static map or (usually)
+        the endpoints service. The result is cached. 
+
+        The URL consists of the protocol and hostname, for example
+        https://api.cloudinsight.alertlogic.com
+        """
+        
         if self._global_endpoint == "map":
             return self.get_mapped_url(service_name, account_id)
         elif re.match(r'^(http|https)://.*$', self._global_endpoint):
@@ -279,10 +290,6 @@ class Session():
             cookies={},
             **kwargs):
 
-        # Make sure we've authenticated
-        if self._token is None:
-            self._authenticate()
-
         # it's too easy to include the AIMS token when pasting debug logs, so redact it in
         # the logging statement.
         headers.update({'x-aims-auth-token': "REDACTED"})
@@ -294,7 +301,7 @@ class Session():
                      f"Cookies: '{cookies}' " +
                      f"Args: '{kwargs}'")
 
-        headers.update({'x-aims-auth-token': self._token})
+        headers.update({'x-aims-auth-token': self.token})
 
         response = self._session.request(
                 method, url,
@@ -359,6 +366,8 @@ class Session():
 
     @property
     def token(self):
+        if self._token is None:
+            self._authenticate()
         return self._token
 
     @property
